@@ -3,8 +3,7 @@
 if [ $(whoami) = wladimir ]; then
   confflags="--prefix=/home/wladimir --with-cuda --disable-gtk-doc"
 elif [ $(whoami) = ds ]; then
-  #confflags="--disable-gtk-doc --enable-orc"
-  confflags="--enable-gtk-doc"
+  confflags="--enable-gtk-doc --disable-orc"
 else
   confflags="--enable-gtk-doc"
 fi
