@@ -60,6 +60,17 @@ schro_async_free (SchroAsync *async)
   schro_free(async);
 }
 
+void
+schro_async_start (SchroAsync *async)
+{
+}
+
+void
+schro_async_stop (SchroAsync *async)
+{
+}
+
+#ifdef unused
 void
 schro_async_run_locked (SchroAsync *async, void (*func)(void *), void *ptr)
 {
@@ -68,12 +79,24 @@ schro_async_run_locked (SchroAsync *async, void (*func)(void *), void *ptr)
   async->task_func = func;
   async->task_priv = ptr;
 }
+#endif
 
+void
+schro_async_run_stage_locked (SchroAsync *async, SchroAsyncStage *stage)
+{
+  SCHRO_ASSERT(async->task_func == NULL);
+
+  async->task_func = stage->task_func;
+  async->task_priv = stage;
+}
+
+#ifdef unused
 int schro_async_get_num_completed (SchroAsync *async)
 {
   if (async->done_priv) return 1;
   return 0;
 }
+#endif
 
 void *schro_async_pull (SchroAsync *async)
 {
@@ -118,6 +141,7 @@ schro_async_wait_locked (SchroAsync *async)
   return TRUE;
 }
 
+#ifdef unused
 void
 schro_async_wait_one (SchroAsync *async)
 {
@@ -128,7 +152,9 @@ schro_async_wait_one (SchroAsync *async)
     async->complete (async->task_priv);
   }
 }
+#endif
 
+#ifdef unused
 void
 schro_async_wait (SchroAsync *async, int min_waiting)
 {
@@ -139,6 +165,7 @@ schro_async_wait (SchroAsync *async, int min_waiting)
     async->complete (async->task_priv);
   }
 }
+#endif
 
 void schro_async_lock (SchroAsync *async)
 {
