@@ -22,10 +22,11 @@ static void
 schro_engine_check_new_sequence_header(SchroEncoder *encoder,
     SchroEncoderFrame *frame)
 {
-  if (encoder->au_frame == -1 ||
+  if (encoder->force_sequence_header ||
       frame->frame_number >= encoder->au_frame + encoder->au_distance) {
     frame->start_sequence_header = TRUE;
     encoder->au_frame = frame->frame_number;
+    encoder->force_sequence_header = FALSE;
   }
 }
 
@@ -55,15 +56,17 @@ schro_engine_code_picture (SchroEncoderFrame *frame,
   frame->picture_number_ref[0] = ref0;
   frame->picture_number_ref[1] = ref1;
 
-  frame->state |= SCHRO_ENCODER_FRAME_STATE_HAVE_GOP;
+  frame->stages[SCHRO_ENCODER_FRAME_STAGE_HAVE_GOP].is_done = TRUE;
   frame->slot = encoder->next_slot++;
 
   if (num_refs > 0) {
+    SCHRO_ASSERT(ref0 >= encoder->au_frame);
     frame->ref_frame[0] = schro_encoder_reference_get (encoder, ref0);
     SCHRO_ASSERT(frame->ref_frame[0]);
     schro_encoder_frame_ref (frame->ref_frame[0]);
   }
   if (num_refs > 1) {
+    SCHRO_ASSERT(ref0 >= encoder->au_frame);
     frame->ref_frame[1] = schro_encoder_reference_get (encoder, ref1);
     SCHRO_ASSERT(frame->ref_frame[1]);
     schro_encoder_frame_ref (frame->ref_frame[1]);
@@ -90,6 +93,7 @@ schro_engine_code_picture (SchroEncoderFrame *frame,
  * Sets up coding parameters for encoding as a completely independent
  * non-ref intra picture.
  */
+#if 0
 static void
 schro_engine_code_intra (SchroEncoderFrame *frame, double weight)
 {
@@ -98,46 +102,116 @@ schro_engine_code_intra (SchroEncoderFrame *frame, double weight)
   frame->picture_weight = weight;
   frame->gop_length = 1;
 }
+#endif
 
 void
-schro_engine_code_IBBBP (SchroEncoder *encoder, int i, int gop_length)
+schro_encoder_pick_refs (SchroEncoderFrame *frame,
+    SchroPictureNumber *ptr_ref0, SchroPictureNumber *ptr_ref1)
 {
-  SchroEncoderFrame *frame;
-  SchroEncoderFrame *f;
-  SchroEncoderFrame *ref2;
-  int j;
+  SchroEncoder *encoder = frame->encoder;
+  SchroPictureNumber ref0;
+  SchroPictureNumber ref1;
+  int i;
 
-  frame = encoder->frame_queue->elements[i].data;
-
-  /* IBBBP */
-  schro_engine_code_picture (frame, TRUE, encoder->intra_ref, 0, -1, -1);
-  encoder->intra_ref = frame->frame_number;
-
-  frame->presentation_frame = frame->frame_number;
-  //frame->picture_weight = 1 + (gop_length - 1) * 0.6;
-  frame->picture_weight = encoder->magic_keyframe_weight;
-  frame->gop_length = gop_length;
-
-  f = encoder->frame_queue->elements[i+gop_length-1].data;
-  schro_engine_code_picture (f, TRUE, encoder->last_ref,
-      1, frame->frame_number, -1);
-
-  f->presentation_frame = frame->frame_number;
-  f->picture_weight = encoder->magic_inter_p_weight;
-  //f->picture_weight += (gop_length - 2) * (1 - encoder->magic_inter_b_weight);
-  encoder->last_ref = encoder->last_ref2;
-  encoder->last_ref2 = f->frame_number;
-  ref2 = f;
-
-  for (j = 1; j < gop_length - 1; j++) {
-    f = encoder->frame_queue->elements[i+j].data;
-    schro_engine_code_picture (f, FALSE, -1,
-        2, frame->frame_number, ref2->frame_number);
-    f->presentation_frame = f->frame_number;
-    if (j == gop_length-2) {
-      f->presentation_frame++;
+  ref0 = SCHRO_PICTURE_NUMBER_INVALID;
+  /* pick the most recent back ref */
+  for(i=0;i<SCHRO_LIMIT_REFERENCE_FRAMES;i++){
+    if (encoder->reference_pictures[i] == NULL) continue;
+    if (encoder->reference_pictures[i]->frame_number < frame->frame_number &&
+        (ref0 == SCHRO_PICTURE_NUMBER_INVALID ||
+         encoder->reference_pictures[i]->frame_number > ref0)) {
+      ref0 = encoder->reference_pictures[i]->frame_number;
+    }
+  }
+  SCHRO_ASSERT(ref0 != SCHRO_PICTURE_NUMBER_INVALID);
+
+  /* pick the earliest forward ref */
+  ref1 = SCHRO_PICTURE_NUMBER_INVALID;
+  for(i=0;i<SCHRO_LIMIT_REFERENCE_FRAMES;i++){
+    if (encoder->reference_pictures[i] == NULL) continue;
+    if (!encoder->reference_pictures[i]->expired_reference &&
+        encoder->reference_pictures[i]->frame_number > frame->frame_number &&
+        (ref1 == SCHRO_PICTURE_NUMBER_INVALID ||
+         encoder->reference_pictures[i]->frame_number < ref1)) {
+      ref1 = encoder->reference_pictures[i]->frame_number;
+    }
+  }
+  if (ref1 == SCHRO_PICTURE_NUMBER_INVALID) {
+    /* if there's no fwd ref, pick an older back ref */
+    for(i=0;i<SCHRO_LIMIT_REFERENCE_FRAMES;i++){
+      if (encoder->reference_pictures[i] == NULL) continue;
+      if (!encoder->reference_pictures[i]->expired_reference &&
+          encoder->reference_pictures[i]->frame_number < ref0 &&
+          (ref1 == SCHRO_PICTURE_NUMBER_INVALID ||
+           encoder->reference_pictures[i]->frame_number > ref1)) {
+        ref1 = encoder->reference_pictures[i]->frame_number;
+      }
+    }
+  }
+
+  *ptr_ref0 = ref0;
+  *ptr_ref1 = ref1;
+}
+
+void
+schro_encoder_pick_retire (SchroEncoderFrame *frame,
+    SchroPictureNumber *ptr_retire)
+{
+  SchroEncoder *encoder = frame->encoder;
+  SchroPictureNumber retire;
+  int n_refs = 0;
+  int i;
+
+  retire = SCHRO_PICTURE_NUMBER_INVALID;
+  /* pick the oldest expired ref */
+  for(i=0;i<SCHRO_LIMIT_REFERENCE_FRAMES;i++){
+    if (encoder->reference_pictures[i] == NULL) continue;
+    n_refs++;
+    if (encoder->reference_pictures[i]->expired_reference &&
+        (retire == SCHRO_PICTURE_NUMBER_INVALID ||
+         encoder->reference_pictures[i]->frame_number < retire)) {
+      retire = encoder->reference_pictures[i]->frame_number;
+    }
+  }
+
+  if (retire == SCHRO_PICTURE_NUMBER_INVALID && n_refs == 3) {
+    /* if we have a full queue, forceably retire something */
+    for(i=0;i<SCHRO_LIMIT_REFERENCE_FRAMES;i++){
+      if (encoder->reference_pictures[i] == NULL) continue;
+      if (retire == SCHRO_PICTURE_NUMBER_INVALID ||
+           encoder->reference_pictures[i]->frame_number < retire) {
+        retire = encoder->reference_pictures[i]->frame_number;
+      }
+    }
+    SCHRO_ASSERT (retire != SCHRO_PICTURE_NUMBER_INVALID);
+  }
+
+  *ptr_retire = retire;
+}
+
+void
+schro_encoder_expire_reference (SchroEncoder *encoder, SchroPictureNumber ref)
+{
+  int i;
+
+  for(i=0;i<SCHRO_LIMIT_REFERENCE_FRAMES;i++){
+    if (encoder->reference_pictures[i] == NULL) continue;
+    if (encoder->reference_pictures[i]->frame_number == ref) {
+      encoder->reference_pictures[i]->expired_reference = TRUE;
+    }
+  }
+}
+
+void
+schro_encoder_expire_refs_before (SchroEncoder *encoder, SchroPictureNumber ref)
+{
+  int i;
+
+  for(i=0;i<SCHRO_LIMIT_REFERENCE_FRAMES;i++){
+    if (encoder->reference_pictures[i] == NULL) continue;
+    if (encoder->reference_pictures[i]->frame_number < ref) {
+      encoder->reference_pictures[i]->expired_reference = TRUE;
     }
-    f->picture_weight = encoder->magic_inter_b_weight;
   }
 }
 
@@ -147,6 +221,9 @@ schro_engine_code_BBBP (SchroEncoder *encoder, int i, int gop_length)
   SchroEncoderFrame *frame;
   SchroEncoderFrame *f;
   int j;
+  SchroPictureNumber ref0;
+  SchroPictureNumber ref1;
+  SchroPictureNumber retire;
 
   frame = encoder->frame_queue->elements[i].data;
 
@@ -154,24 +231,39 @@ schro_engine_code_BBBP (SchroEncoder *encoder, int i, int gop_length)
   frame->gop_length = gop_length;
 
   f = encoder->frame_queue->elements[i+gop_length-1].data;
-  schro_engine_code_picture (f, TRUE, encoder->last_ref,
-      2, encoder->last_ref2, encoder->intra_ref);
-  f->presentation_frame = encoder->last_ref2;
-  f->picture_weight = encoder->magic_inter_p_weight;
-  //f->picture_weight += (gop_length - 1) * (1 - encoder->magic_inter_b_weight);
-  encoder->last_ref = encoder->last_ref2;
-  encoder->last_ref2 = f->frame_number;
+  if (f->start_sequence_header) {
+    schro_encoder_pick_retire (f, &retire);
+
+    schro_engine_code_picture (f, TRUE, retire, 0, -1, -1);
+    f->picture_weight = encoder->magic_keyframe_weight;
+  } else {
+    schro_encoder_pick_retire (f, &retire);
+    schro_encoder_pick_refs (f, &ref0, &ref1);
+
+    schro_engine_code_picture (f, TRUE, retire,
+        (ref1 == SCHRO_PICTURE_NUMBER_INVALID)?1:2, ref0, ref1);
+    f->picture_weight = encoder->magic_inter_p_weight;
+
+    schro_encoder_expire_reference (encoder, encoder->last_ref);
+    encoder->last_ref = f->frame_number;
+  }
 
   for (j = 0; j < gop_length - 1; j++) {
     f = encoder->frame_queue->elements[i+j].data;
-    schro_engine_code_picture (f, FALSE, -1,
-        2, encoder->last_ref, encoder->last_ref2);
+    schro_encoder_pick_refs (f, &ref0, &ref1);
+
+    schro_engine_code_picture (f, FALSE, -1, 2, ref0, ref1);
     f->presentation_frame = f->frame_number;
     if (j == gop_length-2) {
       f->presentation_frame++;
     }
     f->picture_weight = encoder->magic_inter_b_weight;
   }
+
+  f = encoder->frame_queue->elements[i+gop_length-1].data;
+  if (f->start_sequence_header) {
+    schro_encoder_expire_refs_before (encoder, f->frame_number);
+  }
 }
 
 /**
@@ -191,16 +283,13 @@ schro_engine_get_scene_change_score (SchroEncoder *encoder, int i)
   frame1 = encoder->frame_queue->elements[i].data;
   if (frame1->have_scene_change_score) return TRUE;
 
-  /* FIXME Just because it's the first picture in the queue doesn't
-   * mean it's a scene change.  (But likely is.) */
-  if (i == 0) {
+  frame2 = frame1->previous_frame;
+  if (frame2 == NULL) {
     frame1->scene_change_score = 1.0;
     frame1->have_scene_change_score = TRUE;
     return TRUE;
   }
-
-  frame2 = encoder->frame_queue->elements[i-1].data;
-  if (!(frame2->state & SCHRO_ENCODER_FRAME_STATE_ANALYSE)) {
+  if (!(frame2->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done)) {
     return FALSE;
   }
 
@@ -209,8 +298,8 @@ schro_engine_get_scene_change_score (SchroEncoder *encoder, int i)
   luma = frame1->average_luma - 16.0;
   if (luma > 0.01) {
     double mse[3];
-    schro_frame_mean_squared_error (frame1->downsampled_frames[3],
-        frame2->downsampled_frames[3], mse);
+    schro_frame_mean_squared_error (frame1->downsampled_frames[encoder->downsample_levels-1],
+        frame2->downsampled_frames[encoder->downsample_levels-1], mse);
     frame1->scene_change_score = mse[0] / (luma * luma);
   } else {
     frame1->scene_change_score = 1.0;
@@ -218,6 +307,9 @@ schro_engine_get_scene_change_score (SchroEncoder *encoder, int i)
 
   SCHRO_DEBUG("scene change score %g", frame1->scene_change_score);
 
+  schro_encoder_frame_unref (frame1->previous_frame);
+  frame1->previous_frame = NULL;
+
   frame1->have_scene_change_score = TRUE;
   return TRUE;
 }
@@ -269,6 +361,7 @@ init_params (SchroEncoderFrame *frame)
   SchroParams *params = &frame->params;
   SchroEncoder *encoder = frame->encoder;
   SchroVideoFormat *video_format = params->video_format;
+  int i;
 
   params->video_format = &encoder->video_format;
 
@@ -330,8 +423,25 @@ init_params (SchroEncoderFrame *frame)
   schro_params_calculate_mc_sizes (params);
   schro_params_calculate_iwt_sizes (params);
 
+  {
+    int shift = params->transform_depth;
+    if (encoder->enable_dc_multiquant) {
+      params->horiz_codeblocks[0] = params->iwt_luma_width >> (shift + 3);
+      params->vert_codeblocks[0] = params->iwt_luma_height >> (shift + 3);
+    } else {
+      /* This is to work around a bug in the decoder that was fixed 8/2008. */
+      params->horiz_codeblocks[0] = 1;
+      params->vert_codeblocks[0] = 1;
+    }
+    for(i=1;i<params->transform_depth+1;i++){
+      shift = params->transform_depth + 1 - i;
+
+      params->horiz_codeblocks[i] = params->iwt_luma_width >> (shift + 3);
+      params->vert_codeblocks[i] = params->iwt_luma_height >> (shift + 3);
+    }
+  }
+
   if (frame->params.is_noarith) {
-    int i;
     int shift;
 
     params->horiz_codeblocks[0] = 1;
@@ -347,8 +457,14 @@ init_params (SchroEncoderFrame *frame)
   }
 
   params->mv_precision = encoder->mv_precision;
-  //params->have_global_motion = TRUE;
-  params->codeblock_mode_index = 0;
+  if (encoder->enable_global_motion) {
+    params->have_global_motion = TRUE;
+  }
+  if (encoder->enable_multiquant) {
+    params->codeblock_mode_index = 1;
+  } else {
+    params->codeblock_mode_index = 0;
+  }
 }
 
 static double
@@ -421,18 +537,23 @@ schro_encoder_init_frame (SchroEncoderFrame *frame)
   switch (encoder->gop_structure) {
     case SCHRO_ENCODER_GOP_INTRA_ONLY:
       frame->need_downsampling = FALSE;
+      frame->need_upsampling = FALSE;
       frame->need_average_luma = FALSE;
       break;
     case SCHRO_ENCODER_GOP_ADAPTIVE:
     case SCHRO_ENCODER_GOP_BACKREF:
     case SCHRO_ENCODER_GOP_CHAINED_BACKREF:
       frame->need_downsampling = TRUE;
+      frame->need_upsampling = (encoder->mv_precision > 0);
       frame->need_average_luma = TRUE;
+      frame->need_extension = TRUE;
       break;
     case SCHRO_ENCODER_GOP_BIREF:
     case SCHRO_ENCODER_GOP_CHAINED_BIREF:
       frame->need_downsampling = TRUE;
+      frame->need_upsampling = (encoder->mv_precision > 0);
       frame->need_average_luma = TRUE;
+      frame->need_extension = TRUE;
       break;
   }
 }
@@ -454,44 +575,53 @@ schro_encoder_handle_gop_tworef (SchroEncoder *encoder, int i)
   SchroEncoderFrame *f;
   int j;
   int gop_length;
-  schro_bool intra_start;
   double scs_sum;
 
   frame = encoder->frame_queue->elements[i].data;
 
-  SCHRO_ASSERT(!(frame->state & SCHRO_ENCODER_FRAME_STATE_HAVE_GOP));
+  SCHRO_ASSERT(frame->stages[SCHRO_ENCODER_FRAME_STAGE_HAVE_GOP].is_done == FALSE);
 
-  if (frame->busy || !(frame->state & SCHRO_ENCODER_FRAME_STATE_ANALYSE))
+  if (frame->busy || !frame->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done)
     return;
 
-  schro_engine_check_new_sequence_header (encoder, frame);
+  //schro_engine_check_new_sequence_header (encoder, frame);
 
   gop_length = encoder->magic_subgroup_length;
   SCHRO_DEBUG("handling gop from %d to %d (index %d)", encoder->gop_picture,
       encoder->gop_picture + gop_length - 1, i);
 
-  if (i + gop_length >= encoder->frame_queue->n) {
-    if (encoder->end_of_stream) {
-      gop_length = encoder->frame_queue->n - i;
-    } else {
+  if (encoder->end_of_stream) {
+    gop_length = MIN(gop_length, encoder->frame_queue->n - i);
+  }
+
+  //intra_start = frame->start_sequence_header;
+  scs_sum = 0;
+  for (j = 0; j < gop_length; j++) {
+    if (i + j >= encoder->frame_queue->n) {
       SCHRO_DEBUG("not enough pictures in queue");
       return;
     }
-  }
 
-  intra_start = frame->start_sequence_header;
-  scs_sum = 0;
-  for (j = 0; j < gop_length; j++) {
-    /* FIXME set the gop length correctly for IBBBP */
     f = encoder->frame_queue->elements[i+j].data;
 
-    SCHRO_ASSERT(!(f->state & SCHRO_ENCODER_FRAME_STATE_HAVE_GOP));
+    SCHRO_ASSERT(f->stages[SCHRO_ENCODER_FRAME_STAGE_HAVE_GOP].is_done == FALSE);
 
-    if (f->busy || !(f->state & SCHRO_ENCODER_FRAME_STATE_ANALYSE)) {
+    if (f->busy || !f->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done) {
       SCHRO_DEBUG("picture %d not ready", i + j);
       return;
     }
 
+    if (f->start_sequence_header ||
+        f->frame_number >= encoder->au_frame + encoder->au_distance) {
+      f->start_sequence_header = TRUE;
+      if (encoder->open_gop || j == 0) {
+        gop_length = j + 1;
+      } else {
+        gop_length = j;
+      }
+      break;
+    }
+
     schro_engine_get_scene_change_score (encoder, i+j);
 
     schro_dump (SCHRO_DUMP_SCENE_CHANGE, "%d %g %g\n",
@@ -499,29 +629,48 @@ schro_encoder_handle_gop_tworef (SchroEncoder *encoder, int i)
         f->average_luma);
     SCHRO_DEBUG("scene change score %g", f->scene_change_score);
 
-    if (j==0) {
-      if (f->scene_change_score > encoder->magic_scene_change_threshold) {
-        intra_start = TRUE;
+    if (f->scene_change_score > encoder->magic_scene_change_threshold) {
+      if (j == 0) {
+        /* If the first picture of the proposed subgroup is first
+         * picture of a new shot, we want to encode a sequence header
+         * and an I frame. */
+        f->start_sequence_header = TRUE;
+        gop_length = 1;
+        break;
+      } else {
+        /* If there's a shot change in the middle of the proposed
+         * subgroup, terminate the subgroup early.  Also flag that
+         * picture as a new sequence header (not really necessary). */
+        f->start_sequence_header = TRUE;
+        gop_length = j;
       }
-    } else {
+    }
+
+#if 0
       scs_sum += f->scene_change_score;
       if (scs_sum > encoder->magic_scene_change_threshold) {
         /* matching is getting bad.  terminate gop */
         gop_length = j;
       }
-    }
+#endif
   }
 
   SCHRO_DEBUG("gop length %d", gop_length);
 
+  for(j=0;j<gop_length-1;j++){
+    f = encoder->frame_queue->elements[i+j].data;
+    SCHRO_ASSERT(f->start_sequence_header == FALSE);
+  }
+
   if (gop_length == 1) {
-    schro_engine_code_intra (frame, encoder->magic_bailout_weight);
+    schro_engine_code_BBBP (encoder, i, gop_length);
   } else {
-    if (intra_start) {
-      schro_engine_code_IBBBP (encoder, i, gop_length);
-    } else {
-      schro_engine_code_BBBP (encoder, i, gop_length);
-    }
+    schro_engine_code_BBBP (encoder, i, gop_length);
+  }
+
+  f = encoder->frame_queue->elements[i+gop_length-1].data;
+  if (f->start_sequence_header) {
+    encoder->au_frame = f->frame_number;
   }
 
   encoder->gop_picture += gop_length;
@@ -551,7 +700,7 @@ schro_encoder_handle_quants (SchroEncoder *encoder, int i)
 
   frame = encoder->frame_queue->elements[i].data;
 
-  if (frame->busy || !(frame->state & SCHRO_ENCODER_FRAME_STATE_PREDICT)) return FALSE;
+  if (frame->busy || !frame->stages[SCHRO_ENCODER_FRAME_STAGE_MODE_DECISION].is_done) return FALSE;
 
   encoder->quant_slot++;
 
@@ -559,7 +708,7 @@ schro_encoder_handle_quants (SchroEncoder *encoder, int i)
   schro_encoder_choose_quantisers (frame);
   schro_encoder_estimate_entropy (frame);
 
-  frame->state |= SCHRO_ENCODER_FRAME_STATE_HAVE_QUANTS;
+  frame->stages[SCHRO_ENCODER_FRAME_STAGE_HAVE_QUANTS].is_done = TRUE;
 
   return TRUE;
 }
@@ -583,7 +732,8 @@ schro_encoder_handle_gop_backref (SchroEncoder *encoder, int i)
 
   frame = encoder->frame_queue->elements[i].data;
 
-  if (frame->busy || !(frame->state & SCHRO_ENCODER_FRAME_STATE_ANALYSE)) return;
+  if (frame->busy || !frame->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done)
+    return;
 
   schro_engine_check_new_sequence_header (encoder, frame);
 
@@ -603,7 +753,7 @@ schro_encoder_handle_gop_backref (SchroEncoder *encoder, int i)
   for (j = 0; j < gop_length; j++) {
     f = encoder->frame_queue->elements[i+j].data;
 
-    if (f->busy || !(f->state & SCHRO_ENCODER_FRAME_STATE_ANALYSE)) {
+    if (f->busy || !f->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done) {
       SCHRO_DEBUG("picture %d not ready", i + j);
       return;
     }
@@ -668,14 +818,14 @@ schro_encoder_handle_gop_intra_only (SchroEncoder *encoder, int i)
 
   frame = encoder->frame_queue->elements[i].data;
 
-  if (frame->busy || !(frame->state & SCHRO_ENCODER_FRAME_STATE_ANALYSE)) return;
+  if (frame->busy || !frame->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done) return;
 
   schro_engine_check_new_sequence_header (encoder, frame);
 
   SCHRO_DEBUG("handling gop from %d to %d (index %d)", encoder->gop_picture,
       encoder->gop_picture, i);
 
-  if (frame->busy || !(frame->state & SCHRO_ENCODER_FRAME_STATE_ANALYSE)) {
+  if (frame->busy || !frame->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done) {
     SCHRO_DEBUG("picture %d not ready", i);
     return;
   }
@@ -744,6 +894,7 @@ schro_encoder_setup_frame_lossless (SchroEncoderFrame *frame)
   params->xblen_luma = 8;
   params->ybsep_luma = 8;
   params->yblen_luma = 8;
+  schro_params_calculate_mc_sizes (params);
 
   return TRUE;
 }
@@ -763,7 +914,7 @@ schro_encoder_handle_gop_lowdelay (SchroEncoder *encoder, int i)
 
   frame = encoder->frame_queue->elements[i].data;
 
-  if (frame->busy || frame->state != SCHRO_ENCODER_FRAME_STATE_ANALYSE) return;
+  if (frame->busy || !frame->stages[SCHRO_ENCODER_FRAME_STAGE_ANALYSE].is_done) return;
 
   schro_engine_check_new_sequence_header (encoder, frame);
 
