@@ -45,6 +45,7 @@ sort_u8 (uint8_t *d, int n)
   }
 }
 
+#ifdef unused
 /* reference */
 void
 schro_filter_cwmN_ref (uint8_t *d, uint8_t *s1, uint8_t *s2, uint8_t *s3, int n, int weight)
@@ -71,6 +72,7 @@ schro_filter_cwmN_ref (uint8_t *d, uint8_t *s1, uint8_t *s2, uint8_t *s3, int n,
     d[i] = list[(8+weight)/2];
   }
 }
+#endif
 
 void
 schro_filter_cwmN (uint8_t *d, uint8_t *s1, uint8_t *s2, uint8_t *s3, int n, int weight)
@@ -159,6 +161,7 @@ schro_frame_filter_cwmN (SchroFrame *frame, int weight)
 }
 
 
+#ifdef unused
 static void
 schro_frame_component_filter_cwmN_ref (SchroFrameData *comp, int weight)
 {
@@ -197,7 +200,9 @@ schro_frame_component_filter_cwmN_ref (SchroFrameData *comp, int weight)
   schro_free (tmp1);
   schro_free (tmp2);
 }
+#endif
 
+#ifdef unused
 void
 schro_frame_filter_cwmN_ref (SchroFrame *frame, int weight)
 {
@@ -205,6 +210,7 @@ schro_frame_filter_cwmN_ref (SchroFrame *frame, int weight)
   schro_frame_component_filter_cwmN_ref (&frame->components[1], weight);
   schro_frame_component_filter_cwmN_ref (&frame->components[2], weight);
 }
+#endif
 
 
 #if 0
@@ -236,6 +242,7 @@ schro_filter_cwm7 (uint8_t *d, uint8_t *s1, uint8_t *s2, uint8_t *s3, int n)
 }
 #endif
 
+#ifdef unused
 /* FIXME move to schrooil */
 void
 schro_filter_cwm7 (uint8_t *d, uint8_t *s1, uint8_t *s2, uint8_t *s3, int n)
@@ -267,7 +274,9 @@ schro_filter_cwm7 (uint8_t *d, uint8_t *s1, uint8_t *s2, uint8_t *s3, int n)
     }
   }
 }
+#endif
 
+#ifdef unused
 static void
 schro_frame_component_filter_cwm7 (SchroFrameData *comp)
 {
@@ -306,7 +315,9 @@ schro_frame_component_filter_cwm7 (SchroFrameData *comp)
   schro_free (tmp1);
   schro_free (tmp2);
 }
+#endif
 
+#ifdef unused
 void
 schro_frame_filter_cwm7 (SchroFrame *frame)
 {
@@ -314,9 +325,11 @@ schro_frame_filter_cwm7 (SchroFrame *frame)
   schro_frame_component_filter_cwm7 (&frame->components[1]);
   schro_frame_component_filter_cwm7 (&frame->components[2]);
 }
+#endif
 
 
 
+#ifdef unused
 static void
 lowpass_u8 (uint8_t *d, uint8_t *s, int n)
 {
@@ -342,7 +355,9 @@ lowpass_u8 (uint8_t *d, uint8_t *s, int n)
     d[i] = (x + 128)>>8;
   }
 }
+#endif
 
+#ifdef unused
 /* FIXME move to schrooil */
 static void
 lowpass_vert_u8 (uint8_t *d, uint8_t *s, int n)
@@ -360,8 +375,9 @@ lowpass_vert_u8 (uint8_t *d, uint8_t *s, int n)
     d[i] = (x + 128)>>8;
   }
 }
+#endif
 
-
+#ifdef unused
 static void
 schro_frame_component_filter_lowpass (SchroFrameData *comp)
 {
@@ -393,7 +409,9 @@ schro_frame_component_filter_lowpass (SchroFrameData *comp)
 
   schro_free (tmp);
 }
+#endif
 
+#ifdef unused
 void
 schro_frame_filter_lowpass (SchroFrame *frame)
 {
@@ -401,9 +419,10 @@ schro_frame_filter_lowpass (SchroFrame *frame)
   schro_frame_component_filter_lowpass (&frame->components[1]);
   schro_frame_component_filter_lowpass (&frame->components[2]);
 }
+#endif
 
 
-
+#ifdef unused
 static void
 lowpass_s16 (int16_t *d, int16_t *s, int n)
 {
@@ -429,7 +448,9 @@ lowpass_s16 (int16_t *d, int16_t *s, int n)
     d[i] = (x + 128)>>8;
   }
 }
+#endif
 
+#ifdef unused
 /* FIXME move to schrooil */
 static void
 lowpass_vert_s16 (int16_t *d, int16_t *s, int n)
@@ -447,8 +468,10 @@ lowpass_vert_s16 (int16_t *d, int16_t *s, int n)
     d[i] = (x + 128)>>8;
   }
 }
+#endif
 
 
+#ifdef unused
 static void
 schro_frame_component_filter_lowpass_16 (SchroFrameData *comp)
 {
@@ -481,7 +504,9 @@ schro_frame_component_filter_lowpass_16 (SchroFrameData *comp)
 
   schro_free (tmp);
 }
+#endif
 
+#ifdef unused
 void
 schro_frame_filter_lowpass_16 (SchroFrame *frame)
 {
@@ -489,6 +514,7 @@ schro_frame_filter_lowpass_16 (SchroFrame *frame)
   schro_frame_component_filter_lowpass_16 (&frame->components[1]);
   schro_frame_component_filter_lowpass_16 (&frame->components[2]);
 }
+#endif
 
 
 static void
@@ -528,20 +554,20 @@ generate_coeff (double *coeff, double sigma)
 {
   double q;
   double b0, b0inv, b1, b2, b3, B;
-  
+
   if (sigma >= 2.5) {
     q = 0.98711 * sigma - 0.96330;
-  } else { 
+  } else {
     q = 3.97156 - 4.41554 * sqrt (1 - 0.26891 * sigma);
-  } 
-  
+  }
+
   b0 = 1.57825 + 2.44413*q + 1.4281*q*q + 0.422205*q*q*q;
   b0inv = 1.0/b0;
   b1 = 2.44413*q + 2.85619*q*q + 1.26661*q*q*q;
   b2 = -1.4281*q*q - 1.26661*q*q*q;
   b3 = 0.422205*q*q*q;
   B = 1 - (b1 + b2 + b3)/b0;
-  
+
   coeff[0] = B;
   coeff[1] = b1 * b0inv;
   coeff[2] = b2 * b0inv;
@@ -671,6 +697,7 @@ schro_frame_filter_lowpass2 (SchroFrame *frame, double sigma)
 }
 
 
+#ifdef unused
 void
 schro_frame_filter_wavelet (SchroFrame *frame)
 {
@@ -701,27 +728,23 @@ schro_frame_filter_wavelet (SchroFrame *frame)
     schro_wavelet_transform_2d (comp, SCHRO_WAVELET_LE_GALL_5_3, tmp);
 
     for(i=1;i<4;i++){
-      int width;
-      int height;
-      int stride;
-      int16_t *data;
+      SchroFrameData fd;
       int y;
       int cutoff;
 
-      schro_subband_get (tmpframe, component, i, &params, &data, &stride,
-          &width, &height);
+      schro_subband_get_frame_data (&fd, tmpframe, component, i, &params);
       schro_histogram_init (&hist);
 
-      for(y=0;y<height;y++){
-        schro_histogram_add_array_s16 (&hist, OFFSET(data, y*stride),
-            width);
+      for(y=0;y<fd.height;y++){
+        schro_histogram_add_array_s16 (&hist, OFFSET(fd.data, y*fd.stride),
+            fd.width);
       }
 
       cutoff = 100;
-      for(y=0;y<height;y++){
-        int16_t *line = OFFSET(data, stride*y);
+      for(y=0;y<fd.height;y++){
+        int16_t *line = OFFSET(fd.data, fd.stride*y);
         int x;
-        for(x=0;x<width;x++){
+        for(x=0;x<fd.width;x++){
           if (line[x] > -cutoff && line[x] < cutoff) line[x] = 0;
         }
       }
@@ -733,6 +756,7 @@ schro_frame_filter_wavelet (SchroFrame *frame)
   schro_frame_convert (frame, tmpframe);
   schro_frame_unref (tmpframe);
 }
+#endif
 
 
 static double
@@ -834,7 +858,7 @@ schro_frame_filter_adaptive_lowpass (SchroFrame *frame)
   /* good for 2 Mb DVD intra-only rip */
   sigma = -1.0/slope;
   if (sigma > 1.0) {
-    SCHRO_ERROR("enabling filtering (slope %g)", slope);
+    SCHRO_DEBUG("enabling filtering (slope %g)", slope);
 
     schro_frame_filter_lowpass2 (frame, sigma);
   }
