@@ -4,6 +4,7 @@
 #endif
 
 #include <schroedinger/schroutils.h>
+#include <schroedinger/schrotables.h>
 #include <schroedinger/schrodebug.h>
 #include <schroedinger/schro-stdint.h>
 #include <string.h>
@@ -44,6 +45,7 @@ schro_malloc0 (int size)
   return ptr;
 }
 
+#ifdef unused
 void *
 schro_realloc (void *ptr, int size)
 {
@@ -52,6 +54,7 @@ schro_realloc (void *ptr, int size)
 
   return ptr;
 }
+#endif
 
 void
 schro_free (void *ptr)
@@ -122,16 +125,7 @@ muldiv64 (int a, int b, int c)
 int
 schro_utils_multiplier_to_quant_index (double x)
 {
-  int i = 0;
-  
-  x *= x;
-  x *= x;
-  while (x*x > 2) {
-    x *= 0.5;
-    i++;
-  }
-
-  return i;
+  return CLAMP(rint(log(x)/log(2)*4.0),0,60);
 }
 
 
@@ -157,6 +151,7 @@ __schro_quantise (int value, int quant_factor, int quant_offset)
 {
   unsigned int x;
 
+  if (value == 0) return 0;
   if (value < 0) {
     x = (-value)<<2;
     if (x < quant_factor) {
@@ -194,6 +189,38 @@ schro_quantise_s16 (int16_t *dest, int16_t *src, int quant_factor,
   }
 }
 
+void
+schro_quantise_s16_table (int16_t *dest, int16_t *src, int quant_index,
+    schro_bool is_intra, int n)
+{
+  int i;
+  int16_t *table;
+  
+  table = schro_tables_get_quantise_table(quant_index);
+
+  table += 32768;
+
+  for(i=0;i<n;i++){
+    dest[i] = table[src[i]];
+  }
+}
+
+void
+schro_dequantise_s16_table (int16_t *dest, int16_t *src, int quant_index,
+    schro_bool is_intra, int n)
+{
+  int i;
+  int16_t *table;
+  
+  table = schro_tables_get_dequantise_table(quant_index, is_intra);
+
+  table += 32768;
+
+  for(i=0;i<n;i++){
+    dest[i] = table[src[i]];
+  }
+}
+
 void
 schro_dequantise_s16 (int16_t *dest, int16_t *src, int quant_factor,
     int quant_offset, int n)
@@ -211,19 +238,19 @@ schro_dequantise_s16 (int16_t *dest, int16_t *src, int quant_factor,
 
 double
 schro_utils_probability_to_entropy (double x)
-{ 
+{
   if (x <= 0 || x >= 1.0) return 0;
-  
+
   return -(x * log(x) + (1-x) * log(1-x))*INV_LOG_2;
 }
 
 double
 schro_utils_entropy (double a, double total)
-{ 
+{
   double x;
-  
+
   if (total == 0) return 0;
-  
+
   x = a / total;
   return schro_utils_probability_to_entropy (x) * total;
 }
