@@ -37,6 +37,49 @@ schro_video_format_validate (SchroVideoFormat *format)
   return 1;
 }
 
+#ifdef unused
+int
+schro_video_format_compare_new_sequence (SchroVideoFormat *a, SchroVideoFormat *b)
+{
+  if (a->index != b->index ||
+      a->width != b->width ||
+      a->height != b->height ||
+      a->chroma_format != b->chroma_format ||
+      a->interlaced != b->interlaced ||
+      a->top_field_first != b->top_field_first ||
+      a->frame_rate_numerator != b->frame_rate_numerator ||
+      a->frame_rate_denominator != b->frame_rate_denominator ||
+      a->aspect_ratio_numerator != b->aspect_ratio_numerator ||
+      a->aspect_ratio_denominator != b->aspect_ratio_denominator ||
+      a->clean_width != b->clean_width ||
+      a->clean_height != b->clean_height ||
+      a->left_offset != b->left_offset ||
+      a->top_offset != b->top_offset ||
+      a->luma_offset != b->luma_offset ||
+      a->luma_excursion != b->luma_excursion ||
+      a->chroma_offset != b->chroma_offset ||
+      a->chroma_excursion != b->chroma_excursion ||
+      a->colour_primaries != b->colour_primaries ||
+      a->colour_matrix != b->colour_matrix ||
+      a->transfer_function != b->transfer_function) {
+    return FALSE;
+  }
+  return TRUE;
+}
+#endif
+
+#ifdef unused
+int
+schro_video_format_compare (SchroVideoFormat *a, SchroVideoFormat *b)
+{
+  if (!schro_video_format_compare_new_sequence (a, b) ||
+      a->interlaced_coding != b->interlaced_coding) {
+    return FALSE;
+  }
+  return TRUE;
+}
+#endif
+
 int
 schro_video_format_get_bit_depth (SchroVideoFormat *format)
 {
@@ -172,6 +215,34 @@ schro_video_formats[] = {
     2048, 1536, 0, 0,
     256, 3504, 2048, 3584,
     3, 0, 0 },
+  { 17, /* UHDTV 4K-60  */
+    3840, 2160, SCHRO_CHROMA_422,
+    FALSE, TRUE,
+    60000, 1001, 1, 1,
+    3840, 2160, 0, 0,
+    64, 876, 512, 896,
+    0, 0, 0 },
+  { 18, /* UHDTV 4K-50 */
+    3840, 2160, SCHRO_CHROMA_422,
+    FALSE, TRUE,
+    50, 1, 1, 1,
+    3840, 2160, 0, 0,
+    64, 876, 512, 896,
+    0, 0, 0 },
+  { 19, /* UHDTV 8K-60 */
+    7680, 4320, SCHRO_CHROMA_422,
+    FALSE, TRUE,
+    60000, 1001, 1, 1,
+    7680, 4320, 0, 0,
+    64, 876, 512, 896,
+    0, 0, 0 },
+  { 20, /* UHDTV 8K-50 */
+    7680, 4320, SCHRO_CHROMA_422,
+    FALSE, TRUE,
+    50, 1, 1, 1,
+    7680, 4320, 0, 0,
+    64, 876, 512, 896,
+    0, 0, 0 },
 };
 
 /**
@@ -202,51 +273,48 @@ schro_video_format_get_video_format_metric (SchroVideoFormat *format, int i)
 
   std_format = schro_video_formats + i;
 
-  if (format->width != std_format->width) {
-    metric++;
+  /* this is really important because it can't be overrided */
+  if (format->interlaced && 
+      format->top_field_first == std_format->top_field_first) {
+    metric |= 0x8000;
   }
-  if (format->height != std_format->height) {
-    metric++;
+
+  /* things that contribute to MP_DL */
+  if (format->width <= std_format->width &&
+      format->height <= std_format->height) {
+    metric |= 0x800;
   }
-  if (format->interlaced != std_format->interlaced) {
-    metric++;
+  if (format->frame_rate_numerator == std_format->frame_rate_numerator &&
+      format->frame_rate_denominator == std_format->frame_rate_denominator) {
+    metric |= 0x400;
   }
-  if (format->top_field_first != std_format->top_field_first) {
-    metric++;
+  if (format->aspect_ratio_numerator == std_format->aspect_ratio_numerator &&
+      format->aspect_ratio_denominator == std_format->aspect_ratio_denominator) {
+    metric |= 0x200;
   }
-  if (format->frame_rate_numerator != std_format->frame_rate_numerator) {
-    metric++;
+  if (format->colour_matrix == std_format->colour_matrix &&
+      format->colour_primaries == std_format->colour_primaries &&
+      format->transfer_function == std_format->transfer_function) {
+    metric |= 0x100;
   }
-  if (format->frame_rate_denominator != std_format->frame_rate_denominator) {
-    metric++;
+
+  /* things that contribute to VC2_DL */
+  if (format->width == std_format->width &&
+      format->height == std_format->height) {
+    metric |= 0x80;
   }
-  if (format->aspect_ratio_numerator != std_format->aspect_ratio_numerator) {
-    metric++;
+  if (format->interlaced == std_format->interlaced) {
+    metric |= 0x40;
   }
-  if (format->aspect_ratio_denominator != std_format->aspect_ratio_denominator) {
-    metric++;
-  }
-  if (format->left_offset != std_format->left_offset) {
-    metric++;
-  }
-  if (format->top_offset != std_format->top_offset) {
-    metric++;
-  }
-  if (format->clean_width != std_format->clean_width) {
-    metric++;
-  }
-  if (format->clean_height != std_format->clean_height) {
-    metric++;
-  }
-  if (format->colour_matrix != std_format->colour_matrix) {
-    metric++;
-  }
-  if (format->colour_primaries != std_format->colour_primaries) {
-    metric++;
-  }
-  if (format->transfer_function != std_format->transfer_function) {
-    metric++;
+
+#if 0
+  if (format->left_offset == std_format->left_offset &&
+      format->top_offset == std_format->top_offset &&
+      format->clean_width == std_format->clean_width &&
+      format->clean_height == std_format->clean_height) {
+    metric |= 0x40;
   }
+#endif
 
   return metric;
 }
@@ -260,28 +328,26 @@ schro_video_format_get_video_format_metric (SchroVideoFormat *format, int i)
  * function guesses a standard format to use as a starting point for
  * encoding the video format pointed to by @format.
  *
- * FIXME: the function that guesses the best format is poor
- *
  * Returns: an index to the optimal standard format
  */
 SchroVideoFormatEnum
 schro_video_format_get_std_video_format (SchroVideoFormat *format)
 {
   int metric;
-  int min_index;
-  int min_metric;
+  int max_index;
+  int max_metric;
   int i;
 
-  min_index = 0;
-  min_metric = schro_video_format_get_video_format_metric (format, 0);
+  max_index = 0;
+  max_metric = schro_video_format_get_video_format_metric (format, 1);
   for(i=1;i<ARRAY_SIZE (schro_video_formats); i++) {
     metric = schro_video_format_get_video_format_metric (format, i);
-    if (metric < min_metric) {
-      min_index = i;
-      min_metric = metric;
+    if (metric > max_metric) {
+      max_index = i;
+      max_metric = metric;
     }
   }
-  return min_index;
+  return max_index;
 }
 
 typedef struct _SchroFrameRate SchroFrameRate;
@@ -596,19 +662,23 @@ schro_video_format_get_picture_height (SchroVideoFormat *format)
   return format->height;
 }
 
+#ifdef unused
 int
 schro_video_format_get_chroma_width (SchroVideoFormat *format)
 {
   return ROUND_UP_SHIFT(format->width,
       SCHRO_CHROMA_FORMAT_H_SHIFT(format->chroma_format));
 }
+#endif
 
+#ifdef unused
 int
 schro_video_format_get_chroma_height (SchroVideoFormat *format)
 {
   return ROUND_UP_SHIFT(format->height,
       SCHRO_CHROMA_FORMAT_V_SHIFT(format->chroma_format));
 }
+#endif
 
 void
 schro_video_format_get_picture_luma_size (SchroVideoFormat *format,
@@ -631,7 +701,7 @@ schro_video_format_get_picture_chroma_size (SchroVideoFormat *format,
 
 void
 schro_video_format_get_iwt_alloc_size (SchroVideoFormat *format,
-    int *width, int *height)
+    int *width, int *height, int transform_depth)
 {
   int picture_chroma_width;
   int picture_chroma_height;
@@ -640,9 +710,9 @@ schro_video_format_get_iwt_alloc_size (SchroVideoFormat *format,
       &picture_chroma_height);
 
   picture_chroma_width = ROUND_UP_POW2(picture_chroma_width,
-      SCHRO_LIMIT_TRANSFORM_DEPTH);
+      transform_depth);
   picture_chroma_height = ROUND_UP_POW2(picture_chroma_height,
-      SCHRO_LIMIT_TRANSFORM_DEPTH);
+      transform_depth);
 
   *width = picture_chroma_width <<
     SCHRO_CHROMA_FORMAT_H_SHIFT(format->chroma_format);
@@ -650,3 +720,62 @@ schro_video_format_get_iwt_alloc_size (SchroVideoFormat *format,
       SCHRO_CHROMA_FORMAT_V_SHIFT(format->chroma_format);
 }
 
+schro_bool
+schro_video_format_check_MP_DL (SchroVideoFormat *format)
+{
+  SchroVideoFormat base_format;
+
+  if (format->index < 1 || format->index > 20) {
+    return FALSE;
+  }
+
+  schro_video_format_set_std_video_format (&base_format, format->index);
+
+  if (format->width > base_format.width ||
+      format->height > base_format.height) {
+    return FALSE;
+  }
+
+  if (format->frame_rate_numerator != base_format.frame_rate_numerator ||
+      format->frame_rate_denominator != base_format.frame_rate_denominator) {
+    return FALSE;
+  }
+
+  if (format->clean_width != base_format.clean_width ||
+      format->clean_height != base_format.clean_height ||
+      format->left_offset != base_format.left_offset ||
+      format->top_offset != base_format.top_offset) {
+    return FALSE;
+  }
+
+  if (schro_video_format_get_std_signal_range (format) != 2) {
+    return FALSE;
+  }
+
+  if (format->colour_primaries != base_format.colour_primaries ||
+      format->colour_matrix != base_format.colour_matrix ||
+      format->transfer_function != base_format.transfer_function) {
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+schro_bool
+schro_video_format_check_VC2_DL (SchroVideoFormat *format)
+{
+  SchroVideoFormat base_format;
+
+  if (format->index < 1 || format->index > 20) {
+    return FALSE;
+  }
+
+  schro_video_format_set_std_video_format (&base_format, format->index);
+
+  if (memcmp (&base_format, format, sizeof(SchroVideoFormat)) != 0) {
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
